UsbKeyboard

Struct UsbKeyboard 

pub struct UsbKeyboard<L>
where L: KeyboardLayout,
{ /* private fields */ }
Encapsulates HID frame handling, and handles state transitions and key events for USB HID Keyboards

Implementations

Source

impl<L> UsbKeyboard<L>
where L: KeyboardLayout,

pub fn new(layout: L, handle_ctrl: HandleControl) -> UsbKeyboard<L>

Construct USB HID keyboard handler

pub fn reset_state(&mut self)

Reset the USB HID state

We only get changes in key state from the keyboard, so this function resets our internal state.

pub fn handle_report_raw<'kb>( &'kb mut self, report: &UsbBootKeyboardReport, ) -> UsbKeyEventIter<'kb, L>

Process a new USB HID Frame into KeyEvents

You are given an iterator, which will process the incoming report. Do not drop the iterator until it starts returning None.

pub fn handle_report<'kb>( &'kb mut self, report: &UsbBootKeyboardReport, ) -> UsbDecodedKeyIter<'kb, L>

Process a new USB HID Frame into Decoded keys

You are given an iterator, which will process the incoming report. Do not drop the iterator until it starts returning None.